local name, addon = ...;
local L = addon.L

local tip = CreateFrame("GameTooltip")
tip:SetOwner(WorldFrame, "ANCHOR_NONE")

local EPIC, LEGENDARY = L["very fast"], L["extremely fast"]
local RED_LINE = "|cffff2020"
local linecache = {}
for i=1,10 do
	linecache[i] = tip:CreateFontString()
	tip:AddFontStrings(linecache[i], tip:CreateFontString())
end

--====================
-- UTILITY FUNCTIONS
--====================
-- Search the player's skills for one named to match the supplied (localized) string
local function GetSkillLine(skill)
	for i=1, GetNumSkillLines() do
		local name, header, expanded, rank = GetSkillLineInfo(i)
		if header and not expanded then
			ExpandSkillHeader(i) -- Pop these open just in case; not worth reclosing
		elseif name == skill then
			return rank
		end
	end
end

--====================
-- UPDATE FUNCTIONS
--====================
local function USABLE_ZONE(zone)
	return (GetRealZoneText() == zone)
end

local function USABLE_FLYING()
	return (GetWintergraspWaitTime()or GetZoneText()~=L["Wintergrasp"])and IsFlyableArea()
end

local function USABLE_PROFESSION(prof, minimum)
	return (GetSkillLine(prof) and GetSkillLine(prof) >= minimum)
end

local function USABLE_ALWAYS()
	return true
end

local function USABLE_REAGENT(item)
	return (GetItemCount(item) > 0)
end

--====================
-- Lookup tables
--====================
local passengerMounts = {
	[55531] = 1, -- Mekgineer's Chopper
	[60424] = 1, -- Mechano-hog
	[61425] = 2, -- Traveler's Tundra Mammoth (A)
	[61447] = 2, -- Traveler's Tundra Mammoth (H)
	[61465] = 2, -- Grand Black War Mammoth (A)
	[61467] = 2, -- Grand Black War Mammoth (H)
	[61469] = 2, -- Grand Ice Mammoth (H)
	[61470] = 2, -- Grand Ice Mammoth (A)
}

-- This is the set of all very wierd mounts
local exceptions = {
	[25953] = {100, 1, nil, nil, 0, function() return USABLE_ZONE(L["Ahn'Qiraj"]) end}, -- Blue Qiraji Battle Tank
	[26054] = {100, 1, nil, nil, 0, function() return USABLE_ZONE(L["Ahn'Qiraj"]) end}, -- Red Qiraji Battle Tank
	[26055] = {100, 1, nil, nil, 0, function() return USABLE_ZONE(L["Ahn'Qiraj"]) end}, -- Yellow Qiraji Battle Tank
	[26056] = {100, 1, nil, nil, 0, function() return USABLE_ZONE(L["Ahn'Qiraj"]) end}, -- Green Qiraji Battle Tank
	[60024] = {310, nil, 1, nil, 0, USABLE_FLYING}, -- Violet Proto-Drake (tooltip doesn't state proper speed)
	[61451] = {150, nil, 1, nil, 0, function() return USABLE_FLYING() and USABLE_PROFESSION(L["Tailoring"], 410) end}, -- Flying Carpet
	[61309] = {280, nil, 1, nil, 0, function() return USABLE_FLYING() and USABLE_PROFESSION(L["Tailoring"], 425) end}, -- Magnificent Flying Carpet
	[61442] = {280, nil, 1, nil, 0, function() return USABLE_FLYING() and USABLE_PROFESSION(L["Tailoring"], 450) end}, -- Swift Mooncloth Carpet
	[61444] = {280, nil, 1, nil, 0, function() return USABLE_FLYING() and USABLE_PROFESSION(L["Tailoring"], 450) end}, -- Swift Ebonweave Carpet
	[61446] = {280, nil, 1, nil, 0, function() return USABLE_FLYING() and USABLE_PROFESSION(L["Tailoring"], 450) end}, -- Swift Spellweave Carpet
	[44151] = {280, nil, 1, nil, 0, function() return USABLE_FLYING() and USABLE_PROFESSION(L["Engineering"], 375) end}, -- Turbo-Charged Flying Machine
	[44153] = {150, nil, 1, nil, 0, function() return USABLE_FLYING() and USABLE_PROFESSION(L["Engineering"], 350) end}, -- Flying Machine
}

-- These mounts change stats periodically; thus, caching is ineffective. Manually refresh their data as necessary.
-- { canFly, maxFlySpeed }
local variableMounts = {
	[58983] = {false, nil}, -- Big Blizzard Bear
	[54729] = {true, 280}, -- Winged Steed of the Ebon Blade
	[48025] = {true, 280}, -- Headless Horseman's Mount
	[71342] = {true, 310}, -- Big Love Rocket
}

--====================
-- The guts
--====================
local function GenerateMountStats(i)
	tip:ClearLines()
	if not tip:IsOwned(WorldFrame) then tip:SetOwner(WorldFrame, "ANCHOR_NONE") end
	tip:SetHyperlink(GetSpellLink(i))

	local line = linecache[3]:GetText() -- WARNING. ASSUMES ALL MOUNT INFO LIES ON LINE 3.
	local isLand, isFlight, isAquatic, passengers, speed, usable
	passengers = passengerMounts[i] or 0
	
	-- Checking: Flying, Aquatic, Variable, and defaulting to Land
	if strfind(line, L["Outland"]) and strfind(line, L["Northrend"]) then
		isFlight = true
	elseif strfind(line, L["This mount can't move very quickly on land, but she's a great swimmer."]) then
		isAquatic = true
	elseif variableMounts[i] then
		isFlight = variableMounts[i][1] and IsFlyableArea()
		isLand = not isFlight
	else
		isLand = true
	end
	
	-- Set speed based on mount type from above
	if strfind(line, L["Riding skill"]) then
		local ridingSkill = GetSkillLine(L["Riding"])
		if isFlight then
			speed = (ridingSkill == 300 and 280) or 150
		else
			speed = (ridingSkill >= 150 and 100) or 60
		end
	elseif isFlight then
		if variableMounts[i] then
			speed = variableMounts[i][2]
		else
			speed = strfind(line, LEGENDARY) and 310 or (strfind(line, EPIC) and 280 or 150)
		end
	elseif isAquatic then
		speed = 60
	elseif isLand then
		speed = strfind(line, EPIC) and 100 or 60
	else
		speed = 0
		print("Error! Mount #"..i.." appears to have no mount speed!")
	end
	

	-- Set the usability function; for the time being (As of patch 3.3), the mounts that have other usability requirements
	-- either have composite requirements or other issues, qualifying them for a spot in exceptions
	if isFlight then
		usable = USABLE_FLYING
	else
		usable = USABLE_ALWAYS
	end
	return {speed, isLand, isFlight, isAquatic, passengers, usable}
end

-- Cache mount stats in a table
local MountsCache = setmetatable({}, {__index = function(t, i)
	if exceptions[i] then
		t[i] = exceptions[i]
	else
		t[i] = GenerateMountStats(i)
	end
	return t[i]
end})

local critter_usable = setmetatable({}, {__index = function(t, i)
	tip:ClearLines()
	if not tip:IsOwned(WorldFrame) then tip:SetOwner(WorldFrame, "ANCHOR_NONE") end
	tip:SetHyperlink(GetSpellLink(i))
	
	for i=1,tip:NumLines() do
		local item = strfind(linecache[i]:GetText(), L["Reagents:"])
		-- NOTE: Cheating here. The only pets that require reagents as of 3.3 require Snowballs.
		-- Figuring out other items would require stripping escapes from the string.
		if item then
			t[i] = function() return USABLE_REAGENT(17202) end
			return t[i]
		end
	end
	t[i] = USABLE_ALWAYS
	return USABLE_ALWAYS
end})

addon.GetMountStats = function(i)
	if variableMounts[i] then
		return GenerateMountStats(i)
	else
		return MountsCache[i]
	end
end

addon.IsCritterUsable = function(i)
	return critter_usable[i]()
end
